<?php

namespace apexl\Io\modules\display\regions;

use apexl\Io\modules\display\components\TheTopMenuBar;
use apexl\Io\modules\display\includes\region;

class TopBarRegion extends region
{
    public function setRegionWrapper(): void
    {
        $this->regionComponent = $this->componentFactory->make(TheTopMenuBar::class);
    }
}
