<?php

namespace apexl\Io\modules\display\components;

use apexl\Io\modules\component\Component\Component;

readonly class TheLeftSidebar extends Component
{
    public function init(): void
    {
        $this->addClass('col-md-4 col-lg-3 col-xl-2');
    }
}
