<?php

namespace apexl\Io\modules\display\components;

use apexl\Io\modules\component\Component\Component;

readonly class BasicLink extends Component
{
    public function init(): void
    {
        $this->addRoute('/');
        $this->addText('');
    }

    public function addRoute($route): static
    {
        return $this->setProperty('route', $route);
    }

    public function addText($text): static
    {
        return $this->setProperty('text', $text);
    }
}
